import numpy
import pylab
import exceptions
import harkbasenode


def cartesian2polar(xyz):
    """cartesian2polar(x, y, z)
converts coordinate from xyz to r\theta\phi"""
    x, y, z = xyz
    if z == 0:
        raise exceptions.ValueError(
            "cartesian " + str(xyz) + " z must be nonzero.")

    r = pylab.sqrt(sum(map(lambda x: x**2, [x, y, z])))
    theta = pylab.arccos(z/r)
    phi = pylab.arccos(x / pylab.sqrt(sum(map(lambda x: x**2, [x, y]))))
    if y < 0:
        phi *= -1

    return (r, theta, phi)


def r2d(r):
    return 180.0 / numpy.pi * r


class HarkNode(harkbasenode.HarkBaseNode):
    def __init__(self):
        self.outputNames = ("OUTPUT", )
        self.outputTypes = ("prim_int",)

        pylab.ion()
        self.plotx = {}
        self.ploty = {}
        self.winlen = 300

    def calculate(self):
        for src in self.SOURCES:
            if "x" in src.keys():
                self.plotx.setdefault(src["id"], [])
                self.ploty.setdefault(src["id"], [])
                self.plotx[src["id"]].append(self.count)
                self.ploty[src["id"]].append(r2d(cartesian2polar(src["x"])[2]))

        if self.count % 10 == 0:
            for i, srcid in enumerate(self.plotx.keys()):
                if i == 0:
                    pylab.hold(False)
                else:
                    pylab.hold(True)
                pylab.plot(self.plotx[srcid],
                           self.ploty[srcid], "." + "rgb"[srcid % 3])
            pylab.xlim([self.count-self.winlen, self.count])
            pylab.ylim([-180, 180])
            pylab.xlabel("Time [frame]")
            pylab.ylabel("Azimuth [deg]")
            pylab.draw()

        self.outputValues["OUTPUT"] = 0

    def __del__(self):
        pylab.close()
